//
// ReadingDevicesIntoSerialPort.ino
//
// A sample sketch for introduction to Arduino class
//
// ©Danny Swarzman, 2014
//
// This is used to calibrate the sensor analog input.
// It also reads the value the analog input connected to the potentiometer.
//
// It is used to calibrate the sensor.
//
// Using commands in the serial monitor, the user indicates the range of 
// the sensor. The user indicates when the light hitting the sensor is 
// the mininum and at the maximum.
//
// The user also enters the value of the resistor that is connected in 
// series with the photoresistor.
//
// The program displays the values that it reads from the analog pin for 
// maximum and minimum points. It also suggests the ideal size of series 
// resistor.
// 
// It obeys one user command from the serial port: 'a' to toggle
// periodic reporting of 2 analog pins.
//
////////////////////////////////////////////////////////////////////////////////

// Pins A0, A1 and A2 are connected to analog devices.

const int kPotentiometerPin = 0;
const int kPhotoResistorPin = 1;

// Periodically the program displays the values in these sensors in
// the serial port.

const unsigned long kReportPeriodMillis = 5000;
unsigned long reportPeriodExpiry;

// Voltage levels are in the units read by the analog input, 0 to 1023
// To find an actual voltage these would be multiplied by 
// 	VCC / 1023
// where VCC is the supply voltage, nominaly 5V.
// The goal is to get values to plug into another sketch which uses 
// the analog input.

const float kSupplyVoltageLevel = 1023;

int darkVoltageLevel;
int brightVoltageLevel;

long pullupResistance;					// The user enters this value in ohms

boolean periodicFlag;					// Periodic reporting is active	
boolean darkFlag;							// The user has entered dark value
boolean brightFlag;						// The user has entered bright value	
boolean pullupFlag;						//	The user has entered pullup resistor value
boolean enteringResistorValueFlag;	// The waiting for user to enter resistor value

////////////////////////////////////////////////////////////////////////////////

void setup()
{
	Serial.begin(9600);
	showHelp();
	periodicFlag = false;
	darkFlag = false;
	brightFlag = false;
	pullupFlag = false;
	reportPeriodExpiry = millis() + kReportPeriodMillis;
}

void showHelp()
{
	Serial.println ( F("ReadingDevicesIntoSerialPort.ino") );
	Serial.println ( F("Commands" ) );
	Serial.println ( F("p : periodic report" ) );
	Serial.println ( F("s : single report" ) );
	Serial.println ( F("d : remember as dark value" ) );
	Serial.println ( F("b : remember as bright value" ) );
	Serial.println ( F("r : enter value of pullup resisitor" ) );
	Serial.println ( F("? : display this menu" ) );
}

void loop()
{
	loopSerial();
	loopReport();
}
	
void loopSerial()
{
	if ( Serial.available() )
	{
		char clu = Serial.read();
		switch ( clu )
		{
			case 'p':
				periodicFlag = !periodicFlag;
				Serial.print ( F("periodic report " ));
				Serial.println ( periodicFlag ? F("on") : F("off") );
				break;
			case 's':
				oneReport();
				break;
			case 'd':
				darkVoltageLevel = analogRead(kPhotoResistorPin);
				darkFlag = true;
				oneReport();
				break;
			case 'b':
				brightVoltageLevel = analogRead(kPhotoResistorPin);
				brightFlag = true;
				oneReport();
				break;
			case '?':
				showHelp();
				break;
			case 'r':
				pullupResistance = 0;
				enteringResistorValueFlag = true;
				Serial.println ( F("enter resistor value") );
				break;
			default :
				checkEntering ( clu );
				break;
		}
	}
}


// If waiting for user to enter digits for resistor value 
// check the character to see if it is a digit.

void checkEntering ( byte clu )
{
	if ( enteringResistorValueFlag )
	{
		String digits = "0123456789";
		int position = digits.indexOf ( clu );
		if ( position == -1 )
		{
			if ( pullupResistance != 0 )
			{
				// Non-digit at end of sequence
				enteringResistorValueFlag = false;
				pullupFlag = true;
				oneReport();
			}
		} 
		else
		{
			// Digit to add.
			pullupResistance *=10;
			pullupResistance += position;
			pullupFlag = true;
		}
	}
}

void loopReport()
{
	if (periodicFlag && millis() > reportPeriodExpiry )
	{
		reportPeriodExpiry = millis() + kReportPeriodMillis;
		oneReport();
	}
}

void oneReport()
{
	int potentiometer = analogRead(kPotentiometerPin);
	int photoResistor = analogRead(kPhotoResistorPin);
	
	Serial.print ( F("potentiometer " ) );
	Serial.print ( potentiometer );
	Serial.print ( F("   photoResistor " ) );
	Serial.println ( photoResistor );
	if ( darkFlag )
	{
		Serial.print ( F("dark VoltageLevel " ) );
		Serial.println ( darkVoltageLevel );
	}
	else
		Serial.println ( F("dark voltage level has not yet been entered" ));
	
	if ( brightFlag )
	{
		Serial.print ( F("bright VoltageLevel " ) );
		Serial.println ( brightVoltageLevel );
	}
	else
		Serial.println ( F("bright voltage level has not yet been entered" ));
	if ( pullupFlag )
	{
		Serial.print ( F("pullup resistance " ) );
		Serial.println ( pullupResistance );
	}
	else 
		Serial.println( F("value of pullup resistor has not yet been enetered" ));
	if ( brightFlag && darkFlag && pullupFlag )
	{
		Serial.print ( F("range " ) );
		Serial.println ( darkVoltageLevel - brightVoltageLevel );
		if ( pullupFlag )
		{
			float darkRatio = darkVoltageLevel / ( kSupplyVoltageLevel - darkVoltageLevel );
			float darkResistance = pullupResistance * darkRatio;
			float brightRatio = brightVoltageLevel / ( kSupplyVoltageLevel - brightVoltageLevel );
			float brightResistance = pullupResistance * brightRatio;
			float recommendedResistance = sqrt ( darkResistance * brightResistance );
			Serial.print ( F("resistor currently in circuit " ) );
			Serial.println ( pullupResistance );
			Serial.print ( F("dark voltage " ) );
			Serial.println ( darkVoltageLevel );
			Serial.print ( F("calculated dark resistance " ) );
			Serial.println ( darkResistance );
			Serial.print ( F("bright voltage " ) );
			Serial.println ( brightVoltageLevel );
			Serial.print ( F("calculated bright resistance " ) );
			Serial.println ( brightResistance );
			Serial.print ( F("optimal resistance " ) );
			Serial.println ( sqrt ( darkResistance * brightResistance ) );
		}
	}
}



	